//
//  UdpHandler.m
//  LinkRouter
//
//  Created by USRCN on 14-7-4.
//  Copyright (c) 2014年 usrcn. All rights reserved.
//

#import "UdpHandler.h"
#import <ifaddrs.h>
#import <arpa/inet.h>
#import <sys/socket.h>
#import <sys/sysctl.h>
#import <net/if.h>
#import <net/if_dl.h>


#define BIND_PORT 26000
#define BCAST_ADDR @"255.255.255.255"


static UdpHandler * defalutObj;

@implementation UdpHandler

@synthesize socket = _socket;
@synthesize targetPort = _targetPort;

- (id)init{
    if (self == [super init]) {
        [self bind];
    }
    return self;
}

- (void)bind{
    if (_socket) {
        [_socket close];
    }
    _socket = nil;
    _socket = [[GCDAsyncUdpSocket alloc]initWithDelegate:self delegateQueue:dispatch_get_main_queue()];
    [_socket setIPv4Enabled:YES];
    [_socket setIPv6Enabled:NO];
    NSError *error;
    if (![_socket enableBroadcast:YES error:&error]) {
        
        NSLog(@"Error:%@",[error description]);
    }
    if (![_socket beginReceiving:&error]) {
        NSLog(@"Error:%@",[error description]);
    }
    if (![_socket bindToPort:BIND_PORT error:&error]){
        NSLog(@"Error:%@",[error description]);
    }

}

+ (UdpHandler *)defaultUDP{
    if (defalutObj == nil) {
        defalutObj = [[UdpHandler alloc]init];
    }
    return defalutObj;
}

- (void)sendData:(NSData*)data{
    [self bind];
    [_socket sendData:data toHost:BCAST_ADDR port:_targetPort withTimeout:1.6 tag:111];
//    _socket send
}

- (void)getSSIDList{
    // 暂定
    [self bind];
    Byte bytes[] = {0xff,0x00,0x01,0x01,0x02};
    NSData *data = [NSData dataWithBytes:bytes length:5];
    [_socket sendData:data toHost:BCAST_ADDR port:_targetPort withTimeout:-1 tag:110];
    
}

- (void)closeUDPSocket{
    if (![_socket isClosed]){
        [_socket close];
    }
}

- (NSString*)localWifiIPAddr{
    struct ifaddrs *addrs;
    const struct ifaddrs * cursor;
    
    BOOL success = getifaddrs(&addrs) == 0;
    if (success) {
        cursor = addrs;
        while (cursor != NULL) {
            if (cursor->ifa_addr->sa_family == AF_INET && (cursor->ifa_flags & IFF_LOOPBACK) == 0) {
                NSString *name = [NSString stringWithUTF8String:cursor->ifa_name];
                if ([name isEqualToString:@"en0"]) {
                    // Wi-Fi Adapter
                    return [NSString stringWithUTF8String:inet_ntoa(((struct sockaddr_in *)cursor->ifa_addr)->sin_addr)];
                }
            }
            cursor = cursor->ifa_next;
        }
        freeifaddrs(addrs);
    }
    return nil;
}

#pragma mark - GCDAsyncUdpSocket delegate

- (void)udpSocket:(GCDAsyncUdpSocket *)sock didConnectToAddress:(NSData *)address{
    NSString *str = [[NSString alloc]initWithData:address encoding:NSUTF8StringEncoding];
    NSLog(@"has connected to:%@",str);
}

- (void)udpSocket:(GCDAsyncUdpSocket *)sock didReceiveData:(NSData *)data fromAddress:(NSData *)address withFilterContext:(id)filterContext{
    NSString *host = nil;
    uint16_t port = 0;
    [GCDAsyncUdpSocket getHost:&host port:&port fromAddress:address];
    NSArray *arr = [host componentsSeparatedByString:@":"];
    host = [arr lastObject];
    if ([host isEqualToString:[self localWifiIPAddr]]) {
        return;
    }
    NSLog(@"%@received:%@",host,data);
    if (self.delegate && [self.delegate respondsToSelector:@selector(udpReceivedData: fromAddress:)]) {
         [self.delegate udpReceivedData:data fromAddress:address];
    }
}

- (void)udpSocketDidClose:(GCDAsyncUdpSocket *)sock withError:(NSError *)error{
    NSLog(@"udpSocketDidClose error:%@",error.description);
}

@end
